# -*- coding: utf-8 -*-

#  Copyright © 2013-2014  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


class ConnectHelper (object):
    def __init__(self):
        self.handlers = {}
        
    def _connectx(self, gobj, method, signal, func, *args):
        handlers = self.handlers.setdefault(gobj, [])
        handlers.append(method(signal, func, *args))
        
    def connectx(self, gobj, signal, func, *args):
        self._connectx(gobj, gobj.connect, signal, func, *args)
        
    def connectx_after(self, gobj, signal, func):
        self._connectx(gobj, gobj.connect_after, signal, func)
        
    def disconnect_object(self, gobj):
        try:
            handlers = self.handlers[gobj]
        except KeyError:
            return
        while handlers:
            gobj.disconnect(handlers.pop())
        del self.handlers[gobj]
        
    def disconnect_all(self):
        for gobj, handlers in list(self.handlers.items()):
            self.disconnect_object(gobj)
        assert not self.handlers
        
        

